package jclusim.base;

public abstract class Agent {

	private Item carriedItem;
	private Cell currentCell;

	public final void behave() {
		if (isCarryingItem() && !currentCell.hasLaidItem()) {
			if (considerDrop()) {
				drop();
			}

		}
		if (!isCarryingItem() && currentCell.hasLaidItem()) {
			if (considerPickup()) {
				pickup();
			}
		}

		Cell nextCell = chooseNextCell();
		move(nextCell);
	}

	public final void move(Cell nextCell) {
		if (currentCell != null) {
			currentCell.removeAgent(this);
		}

		currentCell = nextCell;
		currentCell.addAgent(this);
	}

	private final void drop()
	{
		assert isCarryingItem() && !(getCurrentCell().hasLaidItem());
		carriedItem.dropOnCell(currentCell);
		carriedItem = null;
	}
	
	private final void pickup()
	{
		assert !(isCarryingItem()) && getCurrentCell().hasLaidItem();
		Item laidObject = currentCell.getLaidItem();
		laidObject.pickUp(this);
		carriedItem = laidObject;
	}
	
	public final boolean isCarryingItem() {
		return carriedItem != null;
	}

	public final Item getCarriedItem() {
		return carriedItem;
	}

	public final Cell getCurrentCell() {
		return currentCell;
	}

	protected abstract Cell chooseNextCell();

	protected abstract boolean considerPickup();

	protected abstract boolean considerDrop();

}