package jclusim.base;

import java.util.Vector;

final public class Cell {
    
    private int x;
    private int y;
    private Environment environment;
    private Item laidItem;
    private Vector<Agent> agents;

    protected Cell(Environment env, int xCoord, int yCoord) {
	x = xCoord;
	y = yCoord;
	laidItem = null;
	agents = new Vector<Agent>();
	environment = env;
    }

    public Item getLaidItem() {
	return laidItem;
    }

    protected void addItem(Item objet) {
	assert !hasLaidItem();

	this.laidItem = objet;
    }

    protected void removeItem() {
	assert hasLaidItem();

	this.laidItem = null;
    }

    protected void addAgent(Agent agent) {
	agents.add(agent);
    }

    protected void removeAgent(Agent agent) {
	agents.remove(agent);
    }

    public boolean hasAgents() {
	return !agents.isEmpty();
    }

    public boolean hasLaidItem() {
	return laidItem != null;
    }

    public Agent getFirstAgent() {
	return agents.elementAt(0);
    }

    public Cell getNeighbor(int dx, int dy) {
	return environment.getCellAt(x + dx, y + dy);
    }

    public Cell[][] getNeighborhood(int radius) {
	return environment.getRegion(x, y, radius);
    }

}
