package jclusim.base;

final public class Environment {

    private int width;
    private int height;
    private Cell[][] cells;

    public Environment(int height, int width) {
	this.height = height;
	this.width = width;

	initCells();
    }

    private void initCells() {
	cells = new Cell[height][width];
	for (int y = 0; y < height; y++) {
	    for (int x = 0; x < width; x++) {
		cells[y][x] = new Cell(this, x, y);
	    }
	}
    }

    public Cell getCellAt(int x, int y) {
	return cells[(y + height) % height][(x + width) % width];
    }

    public int getHeight() {
	return height;
    }

    public int getWidth() {
	return width;
    }

    public Cell[][] getRegion(int x, int y, int size) {
	Cell[][] region = new Cell[2 * size + 1][2 * size + 1];

	for (int yi = -size; yi <= size; yi++) {
	    for (int xi = -size; xi <= size; xi++) {

		region[size + yi][size + xi] = this.getCellAt(x + xi, y + yi);
	    }
	}
	return region;

    }

    public Cell[][] getCells() {
	return cells;
    }

}
