package jclusim.base;

import java.util.Observable;

final public class Experiment extends Observable {

    private IExperimentSettings settings;
    private Environment environment;
    private Agent[] agents;
    private Item[] items;

    private int iterationNumber;

    public Experiment(IExperimentSettings settings) {
	this.settings = settings;
    }

    protected void configure() {

	iterationNumber = 0;

	environment = settings.createEnvironment();

	agents = settings.createAgents();
	items = settings.createItems();

	settings.placeAgentsOnEnvironment(agents, environment);
	settings.placeItemsOnEnvironment(items, environment);

	setChanged();
    }

    public void startExperiment() {

	while (!settings.shouldExperimentStop(this)) {

	    for (int j = 0; j < agents.length; j++) {
		agents[j].behave();
	    }
	    iterationNumber++;

	    setChanged();
	    notifyObservers(iterationNumber);
	}

    }

    public int getIterationNumber() {
	return iterationNumber;
    }

    public Agent[] getAgents() {
	return agents;
    }

    public Item[] getObjects() {
	return items;
    }

    public Environment getEnvironment() {
	return environment;
    }

    public void reset() {
	agents = null;
	items = null;
	environment = null;
	configure();
    }

    public IExperimentSettings getSettings() {
	return settings;
    }

}
