package jclusim.base;

import java.awt.Color;

final public class Item {

    final private double identity[];
    private Cell cell;
    private Agent carryingAgent;

    public Item(double[] values) {
	this.identity = new double[values.length];

	for (int i = 0; i < identity.length; i++) {
	    double value = values[i];
	    value = Math.min(value, 1.0);
	    value = Math.max(value, 0.0);
	    identity[i] = value;
	}
	cell = null;
    }

    public int getIdentityDimension() {
	return identity.length;
    }

    public double getIdentity(int index) {
	return identity[index];
    }

    public int getIdentityMaxIndex() {
	double max = identity[0];
	int indicemax = 0;
	for (int i = 0; i < identity.length; i++) {
	    if (identity[i] > max) {
		max = identity[i];
		indicemax = i;
	    }
	}
	return indicemax;
    }

    public double getIdentityMaxValue() {
	return identity[getIdentityMaxIndex()];
    }


    public double getIdentityNorm() {
	return Math.sqrt(Item.dotProduct(this, this));
    }

    public Cell getCell() {
	return cell;
    }
    
    public static double dotProduct(Item a, Item b) {
	assert a.identity.length == b.identity.length;

	double acc = 0;
	for (int i = 0; i < a.identity.length; i++) {
	    acc = acc + a.identity[i] * b.identity[i];
	}
	return acc;
    }

    public static double distance(Item a, Item b) {
	assert a.identity.length == b.identity.length;

	int n = a.identity.length;
	double sqNorm = 0.0;
	for (int i = 0; i < n; i++) {
	    sqNorm += Math.pow(a.identity[i] - b.identity[i], 2);
	}

	return Math.sqrt(sqNorm);
    }

    public Color getColor() {

	Color color = null;
	if (identity.length == 3) {
	    // RGB Space
	    float r = (float) identity[0];
	    float g = (float) identity[1];
	    float b = (float) identity[2];
	    color = new Color(r, g, b);
	}

	else if (identity.length == 1) {
	    // Gray Scale
	    float value = (float) identity[0];
	    color = new Color(value, value, value);
	}

	else {
	    int maxIndex = getIdentityMaxIndex();
	    int dimension = identity.length;

	    float h = (float) maxIndex / (float) dimension;
	    float s = (float) getIdentityMaxValue();
	    float b = 0.6f;
	    color = Color.getHSBColor(h, s, b);

	}
	return color;
    }

    public Agent getCarryingAgent() {
	return carryingAgent;
    }

    public void dropOnCell(Cell cell) {
	this.cell = cell;
	this.cell.addItem(this);
	this.carryingAgent = null;
    }

    public void pickUp(Agent agent) {
	this.carryingAgent = agent;
	this.cell.removeItem();
	this.cell = null;
    }

    public boolean isLaid() {
	return cell != null;
    }

}
