package jclusim.experiments.simple;

import jclusim.base.Agent;
import jclusim.base.Cell;
import jclusim.base.Environment;
import jclusim.base.Experiment;
import jclusim.base.IExperimentSettings;
import jclusim.base.Item;
import jclusim.util.RandomVariable;

public class SimpleSettings implements IExperimentSettings {

    private static final int NUMBER_OF_ITERATIONS = 200000;
    private static final int DIMENSION = 3;

    public Environment createEnvironment() {
	return new Environment(48, 64);
    }

    public Agent[] createAgents() {
	final int agentNumber = 20;

	SimpleAgent[] agents = new SimpleAgent[agentNumber];
	for (int i = 0; i < agents.length; i++) {
	    agents[i] = new SimpleAgent();
	}
	return agents;
    }

    public void placeAgentsOnEnvironment(Agent[] agents, Environment env) {

	Cell cell = env.getCellAt(32, 24);
	for (int i = 0; i < agents.length; i++) {
	    agents[i].move(cell);
	}
    }

    public Item[] createItems() {
	int itemNumber = 192;

	Item[] items = new Item[itemNumber];
	for (int i = 0; i < items.length; i++) {
	    double[] id = new double[DIMENSION];
	    for (int k = 0; k < DIMENSION; k++) {
		if (k == i % DIMENSION) {
		    id[k] = RandomVariable.drawGaussian(0.6, 0.2);
		} else {
		    id[k] = 0;
		}
	    }
	    items[i] = new Item(id);
	}
	return items;
    }

    public void placeItemsOnEnvironment(Item[] items, Environment env) {
	for (int i = 0; i < items.length; i++) {
	    int row = (i % 16) * 4 + 1;
	    int column = i / 16 * 4 + 1;
	    items[i].dropOnCell(env.getCellAt(row, column));
	}
    }

    public boolean shouldExperimentStop(Experiment exp) {
	return exp.getIterationNumber() >= NUMBER_OF_ITERATIONS;
    }
}
