package jclusim.observer;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;

import jclusim.base.Agent;
import jclusim.base.Cell;
import jclusim.base.Environment;
import jclusim.base.Experiment;

public class EnvironmentObserver extends ExperimentObserver {

    private static final int REFRESH_RATE = 200;
    private static final int UNIT_SIZE = 10;

    private static final int IMAGE_HEIGHT = 480;
    private static final int IMAGE_WIDTH = 640;

    private static final Color BG_COLOR = Color.BLACK;
    private static final Color DEFAULT_AGENT_COLOR = Color.WHITE;

    private BufferedImage bImage;
    private double scaleX;
    private double scaleY;

    @Override
    public synchronized void renderState(Graphics2D g2d) {
	g2d.scale(scaleX, scaleY);
	g2d.drawImage(bImage, 0, 0, null);
    }

    @Override
    public synchronized void updateState(Experiment exp) {
	Graphics2D g2d = bImage.createGraphics();
	Environment environment = exp.getEnvironment();

	Cell[][] cells = environment.getCells();
	for (int y = 0; y < cells.length; y++) {
	    for (int x = 0; x < cells[y].length; x++) {
		Cell cell = cells[y][x];
		int sx = UNIT_SIZE * x;
		int sy = UNIT_SIZE * y;
		if (cell.hasLaidItem()) {
		    g2d.setColor(cell.getLaidItem().getColor());
		    g2d.fillRect(sx, sy, UNIT_SIZE, UNIT_SIZE);
		} else if (cell.hasAgents()) {
		    Agent agent = cell.getFirstAgent();
		    if (agent.isCarryingItem()) {
			g2d.setColor(agent.getCarriedItem().getColor());
		    } else {
			g2d.setColor(DEFAULT_AGENT_COLOR);
		    }
		    g2d.fillOval(sx, sy, UNIT_SIZE, UNIT_SIZE);
		} else {
		    g2d.setColor(BG_COLOR);
		    g2d.fillRect(sx, sy, UNIT_SIZE, UNIT_SIZE);
		}
	    }
	}

    }

    @Override
    public synchronized void resetState(Experiment exp) {
	Environment environment = exp.getEnvironment();
	bImage = new BufferedImage(UNIT_SIZE * environment.getWidth(),
		UNIT_SIZE * environment.getHeight(), BufferedImage.TYPE_INT_RGB);
	scaleX = ((double) IMAGE_WIDTH)
		/ ((double) (UNIT_SIZE * environment.getWidth()));
	scaleY = ((double) IMAGE_HEIGHT)
		/ ((double) (UNIT_SIZE * environment.getHeight()));
    }

    @Override
    public String toString() {
	return "Environment";
    }

    @Override
    public int getCanevasHeight() {
	return IMAGE_HEIGHT;
    }

    @Override
    public int getCanevasWidth() {
	return IMAGE_WIDTH;
    }

    @Override
    public void dumpState(File folder) {
    }

    @Override
    public String summarizeState() {
	return "";
    }

    @Override
    public int getRefreshRate() {
	return REFRESH_RATE;
    }
}