package jclusim.observer;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Observable;
import java.util.Observer;

import javax.imageio.ImageIO;

import jclusim.base.Experiment;

public abstract class ExperimentObserver extends Observable implements Observer {

    private int iterationNumber;

    public abstract int getCanevasHeight();

    public abstract int getCanevasWidth();

    public abstract int getRefreshRate();

    public abstract void renderState(Graphics2D g2d);

    public abstract void resetState(Experiment exp);

    public abstract void updateState(Experiment exp);

    public abstract void dumpState(File folder);

    public abstract String summarizeState();

    public final void takeSnaphot(File baseDir) {
	try {
	    File file = new File(baseDir, toString() + "_" + iterationNumber
		    + ".png");
	    BufferedImage bImage = new BufferedImage(getCanevasWidth(),
		    getCanevasHeight(), BufferedImage.TYPE_INT_RGB);
	    renderState(bImage.createGraphics());
	    ImageIO.write(bImage, "PNG", file);
	} catch (IOException ex) {
	    ex.printStackTrace();
	}

    }

    public final void update(Observable o, Object arg) {
	if (o instanceof Experiment) {
	    Experiment exp = (Experiment) o;
	    int n = exp.getIterationNumber();
	    if (n % getRefreshRate() == 0) {
		iterationNumber = n;
		updateState(exp);
		setChanged();
		notifyObservers();
	    }
	}
    }
}
