package jclusim.util;

import java.util.Random;

/**
 * An utility class for drawing random variables
 */
public class RandomVariable {

    private final static Random random = new Random();

    /**
     * Draw a random variable ~ N(mean,deviation)
     */
    public static double drawGaussian(double mean, double deviation) {
	return ((random.nextGaussian() * deviation) + mean);
    }

    /**
     * Draw a random variable ~ N(0,1)
     */
    public static double drawGaussian() {
	return random.nextGaussian();
    }

    /**
     * Draw a random variable ~ U(0,1)
     */
    public static double drawContinuousUniform() {
	return (random.nextDouble());
    }

    /**
     * Draw a random variable ~ U(a,b)
     */
    public static double drawContinuousUniform(double a, double b) {
	return a + ((b - a) * random.nextDouble());
    }

    /**
     * Pick an element of values (uniformly chosen)
     */
    public static int drawDiscreteUniform(int[] values) {
	return values[random.nextInt(values.length)];
    }

    /**
     * Draw a random variable ~ U(a,b); a and b inclusive
     */
    public static int drawDiscreteUniform(int a, int b) {
	return a + random.nextInt(b - a + 1);
    }

    /**
     * Draw a random index ~ U(0,range); 0 inclusive, range exclusive
     */
    public static int drawIndexUniform(int range) {
	return (random.nextInt(range));
    }

}
