package jclusim.util;

import jclusim.base.Cell;
import jclusim.base.Item;

public abstract class Similarity {

    public abstract double computeSimilarity(Item a, Item b);

    public double computeAveragedSimilarityWithNeigbhorhood(Item candidate, int radius,
	    Cell cell) {
	int nRoi = (2 * radius + 1) * (2 * radius + 1) - 1;
	return computeTotalSimilarityWithNeigbhorhood(candidate, radius, cell) / nRoi;
    }
    
    public double computeTotalSimilarityWithNeigbhorhood(Item candidate, int radius,
	    Cell cell) {
	Cell[][] roi = cell.getNeighborhood(radius);
	double sum = 0;

	for (int i = 0; i < roi.length; i++) {
	    for (int j = 0; j < roi[i].length; j++) {
		if (roi[i][j].hasLaidItem()) {
		    if (!(i == radius && j == radius)) {
			sum = sum
				+ computeSimilarity(candidate, roi[i][j]
					.getLaidItem());
		    }
		}
	    }
	}
	return sum;
    }

    public static final Similarity COSINUS = new CosinusSimilarity();
    public static final Similarity EUCLIDEAN = new EuclideanSimilarity();

    private static class EuclideanSimilarity extends Similarity {

	@Override
	public double computeSimilarity(Item a, Item b) {
	   
	    double norm = Item.distance(a, b);
	    return 1.0 - (norm/a.getIdentityDimension());
	}

	@Override
	public String toString() {
	    return "Euclidean Similarity";
	}

    }

    private static class CosinusSimilarity extends Similarity {

	@Override
	public double computeSimilarity(Item a, Item b) {
	    return Item.dotProduct(a, b) / (a.getIdentityNorm() * b.getIdentityNorm());
	}

	@Override
	public String toString() {
	    return "Cosinus Similarity";
	}

    }
}
